package org.sagebionetworks.repo.web.controller;

import static org.junit.Assert.assertEquals;

import java.util.logging.Logger;

import org.json.JSONObject;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.sagebionetworks.repo.web.UrlHelpers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

/**
 * Unit tests for the Layer CRUD operations exposed by the LayerController with
 * JSON request and response encoding.
 * <p>
 * 
 * Note that test logic and assertions common to operations for all DAO-backed
 * entities can be found in the Helpers class. What follows are test cases that
 * make use of that generic test logic with some assertions specific to layers.
 * <p>
 * 
 * @author deflaux
 */
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = { "classpath:test-context.xml" })
public class LayerAnnotationsControllerTest {
	
	private static final Logger log = Logger
	.getLogger(LayerAnnotationsControllerTest.class.getName());


	@Autowired
	private Helpers helper;
	private JSONObject project;
	private JSONObject dataset;

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		helper.setUp();

		project = helper.testCreateJsonEntity(helper.getServletPrefix() + UrlHelpers.ENTITY,
				DatasetControllerTest.SAMPLE_PROJECT);

		dataset = helper.testCreateJsonEntity(helper.getServletPrefix() + UrlHelpers.ENTITY,
				DatasetControllerTest.getSampleDataset(project.getString("id")));
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
		helper.tearDown();
	}

	/*************************************************************************************************************************
	 * Happy case tests
	 */

	/**
	 * Test method for
	 * {@link org.sagebionetworks.repo.web.controller.LayerAnnotationsController#updateChildEntityAnnotations}
	 * .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testUpdateLayerAnnotations() throws Exception {
		JSONObject newLayer = helper.testCreateJsonEntity(helper.getServletPrefix() + UrlHelpers.ENTITY,
				LayerControllerTest.getSampleLayer(dataset.getString("id")));

		helper.testEntityAnnotations(newLayer.getString("annotations"));
	}

	/*****************************************************************************************************
	 * Not Found Tests
	 */

	/**
	 * Test method for
	 * {@link org.sagebionetworks.repo.web.controller.LayerAnnotationsController#updateChildEntityAnnotations}
	 * .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testGetNonExistentLayerAnnotations() throws Exception {

		// Load up a layer
		JSONObject newLayer = helper
				.testCreateJsonEntity(helper.getServletPrefix()  + UrlHelpers.ENTITY,
						"{\"entityType\":\"org.sagebionetworks.repo.model.Data\", \"name\":\"MouseCross\", \"type\":\"C\",  \"description\": \"foo\"," +
						" \"parentId\":\"" +dataset.getString("id")+"\"}");
		
		// Get our empty annotations container
		JSONObject annotations = helper.testGetJsonEntity(newLayer
				.getString("annotations"));

		// Delete our layer
		helper.testDeleteJsonEntity(newLayer.getString("uri"));

		JSONObject error = helper.testGetJsonEntityShouldFail(annotations
				.getString("uri"), HttpStatus.NOT_FOUND);
		assertEquals(
				"The resource you are attempting to access cannot be found",
				error.getString("reason"));

	}

}
